
//
// wexcept.h: This header declares the 
//            exception classes
//
// This file is part of the Winterdom library
// Copyright (C) 2000-2001, Tomas Restrepo (tomasr@mvps.org)
//            
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef WEXCEPT_H__INCLUDED
#define WEXCEPT_H__INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "wctypes.h"


namespace Winterdom {
   namespace Support {


      //
      // Exception
      //
      //  Base class of our exception classes
      //
      class Exception
      {
      public:
         virtual ustring ToString() const =0;
      };


      //
      // Win32Exception
      //
      //  Represents an exception thrown
      //  as a result of a Win32 error.
      //  It is not related to SEH exceptions!
      //
      class Win32Exception : public Exception
      {
      public:
         Win32Exception ( DWORD code ) : m_errorCode ( code )
         {
         }

         DWORD ErrorCode() const
         {
            return m_errorCode;
         }

         virtual ustring ToString() const
         {
            TCHAR * buffer = 0;
            FormatMessage ( 
                  FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM
                  | FORMAT_MESSAGE_IGNORE_INSERTS,
                  NULL, m_errorCode, GetUserDefaultLangID( ),
                  (LPTSTR)&buffer, 0, NULL
               );

            ustring str = _T("");
            if ( buffer != 0 ) {
               str = buffer;
               LocalFree ( buffer );
            }
            return str;
         }
      protected:
         DWORD m_errorCode;
      };


      //
      // GenericException
      //
      //  Represents a generic exception, that
      //  carries no extra info except an
      //  error string
      //
      class GenericException
      {
      public:
         // from a string
         GenericException ( const TCHAR * str ) 
            : m_errorString(str) 
         {
         }
         // from a resource string
         GenericException ( HINSTANCE hInst, UINT strId )
         {
            const int size = 1024;
            TCHAR buffer[size]; buffer[0] = '\0';
            LoadString ( hInst, strId, buffer, size );
            m_errorString = buffer;
         }

         virtual ustring ToString() const
         {
            return m_errorString;
         }
      protected:
         ustring m_errorString;
      };

   } // namespace Support
} // namespace Winterdom
#endif // WEXCEPT_H__INCLUDED
